// DetectBlobs.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include "BlobResult.h"

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>


int _tmain(int argc, _TCHAR* argv[])
{
	// Initialise 	
	//std::string filepath = "spots.bmp";	
	std::string filepath = "spots2.bmp";	
	CBlobResult blobs;
	int num_blobs = 0;

	// Load grayscale version of coloured input image
	IplImage* original = cvLoadImage( filepath.c_str() );

    IplImage* grayscale = cvLoadImage( filepath.c_str(),
                                       CV_LOAD_IMAGE_GRAYSCALE );

	// Check bitmap image exists
	assert( grayscale );			

	// Create IplImage struct for a black and white (binary) image
	IplImage* img_bw = cvCreateImage( cvGetSize( grayscale ),
                                      IPL_DEPTH_8U,
                                      1 );

	// Use thresholding to convert grayscale image into binary
    cvThreshold( grayscale,           // source image
				 img_bw,              // destination image
				 40,                  // threhold val.
				 255,                 // max. val
				 CV_THRESH_BINARY );  // binary type ); 

	// Create IplImage struct for inverted black and white image	   
	IplImage* img_bw_cpy = cvCloneImage( img_bw );	
				
	// Find connected components using cvBlobsLib	
	blobs = CBlobResult( img_bw, 0, 255 );

    // Exclude all blobs smaller than the given value 
    // The bigger the last parameter, the bigger the blobs need
    // to be for inclusion
    blobs.Filter( blobs,
                  B_EXCLUDE,
                  CBlobGetArea(),
                  B_GREATER,
                  3 );

	num_blobs = blobs.GetNumBlobs(); 

	// Find connected components using OpenCV
	CvSeq* seq;
	CvMemStorage* storage = cvCreateMemStorage( 0 );
    cvClearMemStorage( storage );

	// cvFindContours the 12 + 1 extra object for white backgrounds
	// and black spots, hence subtract 1
	num_blobs = cvFindContours( img_bw, 
								storage, 
								&seq, 
								sizeof( CvContour ), 
								CV_RETR_LIST, 
								CV_CHAIN_APPROX_NONE, 
								cvPoint( 0, 0 ) ) - 1;
	
	// Display the input / output windows and images
	cvNamedWindow( "original" );	
	cvShowImage( "original", original );	

	cvNamedWindow( "grayscale" );	
	cvShowImage( "grayscale", grayscale );		

	cvNamedWindow( "black_and_white" );	
	cvShowImage( "black_and_white", img_bw_cpy );			

	// Wait for user key press and then tidy up
	cvWaitKey(0);
    
	cvReleaseImage( &original );
    cvReleaseImage( &grayscale );	
	cvReleaseImage( &img_bw );	
	cvReleaseImage( &img_bw_cpy );
	    
    cvDestroyWindow( "greyscale" );    
	cvDestroyWindow( "black_and_white" );    
	cvDestroyWindow( "inverted" );

	return 0;
}

